#!/usr/bin/env python
# coding: utf-8

# Plot_time_series_phi2.ipynb
# 
# Generates time series plots of reconstructed Phi2 (PhiPSII) images for each sample.

# In[1]:


import contextlib
import cv2
import glob
import numpy as np
import os
import matplotlib.pyplot as plt
import re
import shutil


# In[2]:


# split lists in chunks of a given size
def split(list, chunk_size):
  for i in range(0, len(list), chunk_size):
    yield list[i:i + chunk_size]

# read, resize and add measurement number to an image    
def read_resize_annotate(img_path, textcolor, high_qual = False):
    round_id = re.search(r'35-\d+', img_path).group(0).split('-')[1]
    img = cv2.imread(img_path)
    if high_qual:
        if textcolor == 'white':
            img = cv2.putText(img, '{}'.format(round_id), (300, 400), cv2.FONT_HERSHEY_SIMPLEX, 8, (255,255,255), 8) # image, text, coordinates, font, font scale, color, thickness
        elif textcolor == 'black':
            img = cv2.putText(img, '{}'.format(round_id), (300, 400), cv2.FONT_HERSHEY_SIMPLEX, 8, (0,0,0), 8) # image, text, coordinates, font, font scale, color, thickness
    else:
        img = cv2.resize(img, dsize = (0,0), fx = 0.2, fy = 0.2) # will create an image with 80% smaller axes
        if textcolor == 'white':
            img = cv2.putText(img, '{}'.format(round_id), (60, 90), cv2.FONT_HERSHEY_SIMPLEX, 2, (255,255,255), 2) # image, text, coordinates, font, font scale, color, thickness
        elif textcolor == 'black':
            img = cv2.putText(img, '{}'.format(round_id), (60, 90), cv2.FONT_HERSHEY_SIMPLEX, 2, (0,0,0), 2) # image, text, coordinates, font, font scale, color, thickness
    return img

# create an empty white or black image
def empty_image(img_name, color, high_qual = False):
    round_id = img_name.split('-')[-1]
    if color == 'w':
        img = 255 * np.ones(shape=(1440, 1920, 3), dtype=np.uint8)
    if color == 'b':
        img = 0 * np.ones(shape=(1024, 1360, 3), dtype=np.uint8)
    if high_qual:
        if textcolor == 'white':
            img = cv2.putText(img, '{}'.format(round_id), (300, 400), cv2.FONT_HERSHEY_SIMPLEX, 8, (255,255,255), 8) # image, text, coordinates, font, font scale, color, thickness
        elif textcolor == 'black':
            img = cv2.putText(img, '{}'.format(round_id), (300, 400), cv2.FONT_HERSHEY_SIMPLEX, 8, (0,0,0), 8) # image, text, coordinates, font, font scale, color, thickness
    else:
        img = cv2.resize(img, dsize = (0,0), fx = 0.2, fy = 0.2) # will create an image with 80% smaller axes
        if textcolor == 'white':
            img = cv2.putText(img, '{}'.format(round_id), (60, 90), cv2.FONT_HERSHEY_SIMPLEX, 2, (255,255,255), 2) # image, text, coordinates, font, font scale, color, thickness
        elif textcolor == 'black':
            img = cv2.putText(img, '{}'.format(round_id), (60, 90), cv2.FONT_HERSHEY_SIMPLEX, 2, (0,0,0), 2) # image, text, coordinates, font, font scale, color, thickness
    return img


# In[3]:


# set directories
data_path = "C:\\Users\\garas001\\OneDrive - Wageningen University & Research\\PhD\\_Year 4\\Chapter 4 - Natural Variation B8\\1_Fluorescence analysis\\" + "single_images\\"
int_path = 'C:\\Users\\garas001\\OneDrive - Wageningen University & Research\\PhD\\_Year 4\\Chapter 4 - Natural Variation B8\\1_Fluorescence analysis\\tmp\\'
out_path = 'C:\\Users\\garas001\\OneDrive - Wageningen University & Research\\PhD\\_Year 4\\Chapter 4 - Natural Variation B8\\1_Fluorescence analysis\\time_series\\'


# In[4]:


# make sure we have the correct output and intermediate directories
if not os.path.exists(out_path):
    os.makedirs(out_path)
os.chdir(out_path)

if not os.path.exists(int_path):
    os.makedirs(int_path)


# In[5]:


# create tray numbers in the right format
tray_nrs  = range(1,181,1)
tray_nrs = [str(item).zfill(3) for item in tray_nrs]

# remove trays in which plants did not grow
bad_trays = bad_trays = [9, 21, 33, 46, 52, 59, 66, 80, 82, 89, 94, 95, 99, 100, 113, 117, 127, 128, 129, 142, 144, 148, 154, 155, 167, 169, 173, 174]
bad_trays_nrs = [str(item).zfill(3) for item in bad_trays]
tray_nrs = [x for x in tray_nrs if x not in bad_trays_nrs]


# In[ ]:


#len(tray_nrs) == (180 - len(bad_trays))


# In[6]:


# development
#tray_nrs = ['001']

# do we want large, high-quality images? (ideal for publication)
hq = False
# specify the parameter!
param = 'phiPSII_hist'

if re.search(r'fvfm', param):
    # these are the rounds corresponding to Fv/Fm measurements
    rounds = [3, 6, 9, 12, 15, 18, 23, 27, 31, 35, 39, 43, 47, 51, 55, 59, 63, 67, 71, 75, 79, 83, 87, 90, 91] # notice the last round is missing to make nr of rounds divisible by 5
elif re.search(r'phiPSII', param):
        rounds = [1, 2, 4, 5, 7, 8, 10, 11, 13, 14, 16, 17, 22, 24, 26, 28, 30, 32, 34, 36, 38, 40, 42, 44, 46, 48, 50, 52, 54, 56, 58, 60, 62, 64, 66, 68, 70, 72, 74, 76, 78, 80, 82, 84, 86]

if re.search(r'hist', param):
    textcolor = 'black'
    figcolor = 255
else:
    textcolor = 'white'
    figcolor = 0
    
for nr in tray_nrs:
    selected_imgs = sorted(glob.glob('{}*{}*{}.jpg'.format(data_path, nr, param)), key=os.path.getmtime) # this will order images based on creation time, so order rounds in a meaningful way
    if not selected_imgs:
        print('no images found for tray {}'.format(nr))
        continue
    if re.search(r'fvfm', param):
        selected_imgs = selected_imgs[:-1] # remove very last measurement to make arrangement easier
    
    print('processing {} images from tray {}'.format(len(selected_imgs), nr))  
    
    img_list = [0] * len(rounds)
    for i in range(0, len(rounds)):
        r = rounds[i]
        r_withimg = [int(re.search(r'-\d+-', x).group().split('-')[1]) for x in selected_imgs]
        if re.search(r'hist', param):
            if r not in r_withimg:
                empty_img = figcolor * np.ones(shape=(1024, 1360, 3), dtype=np.uint8)
                tray_name = selected_imgs[0].split('\\')[-1].split('-')[2]
                img_name = '-'.join(['35', str(r), tray_name, param]) + '.jpg'
                with contextlib.suppress(ValueError):
                    plt.hist(empty_img)
                plt.savefig(int_path+img_name, dpi=300)
                plt.clf()
                print('\tcreating empy figure {}'.format(img_name))
                img_list[i] = int_path+img_name
            else:
                img_list[i] = shutil.copyfile(selected_imgs[r_withimg.index(r)], int_path + selected_imgs[r_withimg.index(r)].split('\\')[-1])
        else:
            if r not in r_withimg:
                empty_img = figcolor * np.ones(shape=(1024, 1360, 3), dtype=np.uint8)
                tray_name = selected_imgs[0].split('\\')[-1].split('-')[2]
                img_name = '-'.join(['35', str(r), tray_name, param]) + '.jpg'
                plt.imshow(empty_img, vmin = 0, vmax = 0.8)
                plt.savefig(int_path+img_name, dpi=300)
                plt.clf()
                print('\tcreating empy figure {}'.format(img_name))
                img_list[i] = int_path+img_name
            else:
                img_list[i] = shutil.copyfile(selected_imgs[r_withimg.index(r)], int_path + selected_imgs[r_withimg.index(r)].split('\\')[-1])
    
    # generate rows of final image
    allimg_list_split = list(split(img_list, 5)) # create sub-lists
    concats_h = []
    for row in range(0, len(list(allimg_list_split)), 1):
        imgs_loaded = [read_resize_annotate(img, textcolor, high_qual = hq) for img in allimg_list_split[row]] # load and process images from each sub-list
        concats_h.append(cv2.hconcat(imgs_loaded)) # add list of images to master list (each element will be a row of the final image)
    
    # concatenate rows vertically
    concat_vh = cv2.vconcat(concats_h) # create final image by vertically concatenating the various rows
    
    # save image
    if hq:
        cv2.imwrite(out_path + 'Tray_{}_AllImages_{}_concat_HQ.jpg'.format(nr, param), concat_vh, [cv2.IMWRITE_JPEG_QUALITY, 100]) # write out image
    else:
        cv2.imwrite(out_path + 'Tray_{}_AllImages_{}_concat.jpg'.format(nr, param), concat_vh, [cv2.IMWRITE_JPEG_QUALITY, 100]) # write out image
    
    # clerar the intermediate folder and re-create it
    shutil.rmtree(int_path)
    os.mkdir(int_path)
    
print('All done')

